#!usr/bin/env python
# -*- coding: utf8 -*-
#
# -----------------------------------------------------------------------------
#   File: annotations_functions.py (as part of project URUMETRICS-CODE)
#   Created: 08/09/2022 12:42
#   Last Modified: 08/09/2022 12:42
# -----------------------------------------------------------------------------
#   Author: William N. Havard
#           Postdoctoral Researcher
#
#   Mail  : william.havard@ens.fr / william.havard@gmail.com
#  
#   Institution: ENS / Laboratoire de Sciences Cognitives et Psycholinguistique
#
# ------------------------------------------------------------------------------
#   Description: 
#       • 
# -----------------------------------------------------------------------------

import argparse
import os

from conversational_settings import uru_conversations
from utils import staticvariable
from utils_annotations import _compute_file_acoustic_annotation


@staticvariable(BASE_SETS=['vtc'])
def conversations_annotations(recording_filename, segments, project_path, parser_args=None):
    return uru_conversations.get_interactional_sequences(segments).to_dataframe()


@staticvariable(BASE_SETS=['vtc'], RAW_RECORDING_AVAILABLE=True)
def acoustic_annotations(recording_filename, segments, project_path, parser_args):
    parser = argparse.ArgumentParser()
    parser.add_argument('--target-sr', required=True, default=16_000, type=int,
                        help='Audio files sampling rate.')

    args = parser.parse_args(parser_args)
    return _compute_file_acoustic_annotation(
                audio_path=os.path.join(project_path, 'recordings', 'raw', recording_filename),
                audio_segments=segments,
                target_sr=args.target_sr)
