#!usr/bin/env python
# -*- coding: utf8 -*-

# -----------------------------------------------------------------------------
#   File: custom_converters.py (as part of project URUMETRICS)
#   Created: 28/07/2022 11:10
#   Last Modified: 28/07/2022 11:10
# -----------------------------------------------------------------------------
#   Author: William N. Havard
#           Postdoctoral Researcher
#
#   Mail  : william.havard@ens.fr / william.havard@gmail.com
#  
#   Institution: ENS / Laboratoire de Sciences Cognitives et Psycholinguistique
#
# ------------------------------------------------------------------------------
#   Description: 
#       • 
# -----------------------------------------------------------------------------

import pandas as pd
from ChildProject.annotations import AnnotationConverter


class FilteredCsvConverter(AnnotationConverter):
    FORMAT = 'filtered_csv'

    @staticmethod
    def convert(filename: str, source_file: str = "", **kwargs) -> pd.DataFrame:
        """
        takes a csv file to import containing a column 'recording_filename', checks that
        there is a unique recording_filename inside or uses a filter to only keep one,
        warns otherwise
        :param filename: csv file to import
        :type filename: str or path like object
        :param source_file: recording to keep (keep the line if contains this in the name)
        :type source_file: str
        """
        df = pd.read_csv(filename)

        n_recordings = len(df["recording_filename"].unique())
        if  n_recordings > 1 and not source_file:
            print(
                f"""WARNING: {filename} contains annotations from {n_recordings} different audio files, """
                """but no filter was specified which means all of these annotations will be imported.\n"""
                """as if they belonged to the same recording. Please make sure this is the intended behavior """
                """(it probably isn't)."""
            )

        if source_file:
            df = df[df["recording_filename"].str.contains(source_file)]

        return df
